import ruamel.yaml
import re
from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from dodo_commands.framework.config import look_up_key


def _args():
    parser = ArgumentParser(description="Print the full configuration.")
    parser.add_argument('key', nargs='?')
    args = Dodo.parse_args(parser)
    if args.key and args.key.startswith('/'):
        args.key = args.key[1:]
    return args


if Dodo.is_main(__name__):
    args = _args()
    if args.key:
        print("%s" % ruamel.yaml.round_trip_dump(
            look_up_key(Dodo.config, args.key, '')
        ))
    else:
        content = re.sub(
            r'^([0-9_A-Z]+\:)$',
            r'\n\1',
            ruamel.yaml.round_trip_dump(Dodo.config),
            flags=re.MULTILINE
        )
        print(
            re.sub(
                r'^\n\n',
                r'\n',
                content,
                flags=re.MULTILINE
            )
        )



from dodo_commands.framework import Dodo, CommandError
from dodo_commands.framework.util import remove_trailing_dashes
import argparse
import os


def _dodo_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'name',
        choices=Dodo.get_config('/DOCKER/images').keys(),
        help='Key to look up in /DOCKER/images'
    )
    parser.add_argument(
        'build_args',
        nargs=argparse.REMAINDER,
        help='Extra args to pass to docker build'
    )
    return Dodo.parse_args(parser)


def _convert_dodo_args():
    dodo_args = _dodo_args()
    gc = Dodo.create_get_config(dodo_args.__dict__)
    args = argparse.Namespace()
    args.build_dir = gc(
        key='/DOCKER/images/{name}/build_dir',
        default='.'
    )
    args.docker_file = gc(
        key='/DOCKER/images/{name}/docker_file',
        default='Dockerfile'
    )
    args.extra_dirs = gc(
        key='/DOCKER/images/{name}/extra_dirs',
        default=[]
    )
    args.docker_image = gc(
        key='/DOCKER/images/{name}/image'
    )
    args.build_args = dodo_args.build_args
    return args


def _args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--build-dir',
        default='.'
    )
    parser.add_argument(
        '--docker-filename',
        default='Dockerfile'
    )
    parser.add_argument(
        '--extra_dir',
        action='append',
        dest='extra_dirs'
    )
    parser.add_argument(
        'docker_image'
    )
    parser.add_argument(
        'build_args',
        nargs=argparse.REMAINDER
    )
    return Dodo.parse_args(parser)


def _copy_extra_dirs(local_dir, extra_dirs):
    # todo: convert extra dirs config mapping into a list of key=val
    for extra_dir in (extra_dirs or []):
        extra_dir_name, extra_dir_path = extra_dir.split('=')
        local_path = os.path.join(local_dir, extra_dir_name)
        if os.path.exists(local_path):
            raise CommandError(
                "Cannot copy to existing path: " + local_path
            )
        if not os.path.exists(extra_dir_path):
            raise CommandError(
                "Cannot copy from non-existing path: " + extra_dir_path
            )
        Dodo.runcmd(['cp', '-rf', extra_dir_path, local_path])


def _remove_extra_dirs(local_dir, extra_dirs):
    for extra_dir in (extra_dirs or []):
        extra_dir_name, extra_dir_path = extra_dir.split('=')
        local_path = os.path.join(local_dir, extra_dir_name)
        Dodo.runcmd(['rm', "rm", "-rf", local_path])


if Dodo.is_main(__name__, safe=False):
    args = (
        _convert_dodo_args()
        if Dodo.command_name else
        _args()
    )

    _copy_extra_dirs(args.build_dir, args.extra_dirs)

    try:
        Dodo.runcmd(
            [
                "docker",
                "build",
                "-t",
                args.docker_image,
                "-f",
                args.docker_file,
            ] + remove_trailing_dashes(args.build_args) +
            [
                ".",
            ],
            cwd=args.build_dir
        )
    finally:
        _remove_extra_dirs(args.build_dir, args.extra_dirs)



from argparse import ArgumentParser
from dodo_commands.framework import Dodo
from dodo_commands.framework import get_version
from dodo_commands.framework.util import bordered
from semantic_version import Version
import os
import ruamel.yaml
import sys


def _args():
    parser = ArgumentParser(
        description='Compare configuration version to '
                    'version in original project config file.'
    )
    parser.add_argument(
        '--dodo',
        dest='check_dodo',
        action='store_true',
        help=(
            "Check that the installed dodo commands version satisfies the "
            "minimal dodo commands version in the dodo config "
            "(/ROOT/required_dodo_commands_version)"
        )
    )
    parser.add_argument(
        '--config',
        dest='check_config',
        action='store_true',
        help=(
            "Check that the version field in the local dodo commands config "
            "(/ROOT/version) is up-to-date with the version in the shared "
            "dodo commands config."
        )
    )
    args = Dodo.parse_args(parser)
    return args


def _get_root_config_field(config_filename, field_name):
    with open(config_filename) as f:
        config = ruamel.yaml.round_trip_load(f.read())
    return config.get("ROOT", {}).get(field_name, "")


def _partial_sem_version(version):
    v = Version(version)
    return Version("%s.%s" % (v.major, v.minor), partial=True)


def _config_filename():
    return os.path.join(
        Dodo.get_config("/ROOT/project_dir", ""),
        "dodo_commands",
        "res",
        "config.yaml"
    )


def check_dodo_commands_version():  # noqa
    required_version = _get_root_config_field(
        _config_filename(), 'required_dodo_commands_version'
    )
    if required_version:
        actual_version = get_version()
        if Version(actual_version) < Version(required_version):
            sys.stdout.write(bordered(
                'The dodo_commands package needs to be upgraded (%s < %s). Tip: use "dodo upgrade"'
                % (
                    actual_version,
                    required_version,
                ),
            ))
            sys.stdout.write('\n')


def check_config_version():  # noqa
    project_dir = Dodo.get_config("/ROOT/project_dir", "")
    original_file = os.path.join(
        project_dir, "dodo_commands", "default_project", "config.yaml"
    )
    if not os.path.exists(original_file):
        return

    original_version = _get_root_config_field(original_file, 'version')
    if not original_version:
        sys.stderr.write(
            "No version found in original file %s\n" % original_file
        )
        return

    copied_file = os.path.join(project_dir, "dodo_commands", "res", "config.yaml")
    copied_version = _get_root_config_field(copied_file, 'version')
    if not copied_version:
        sys.stderr.write(
            "No version found in user managed file %s\n" % copied_file
        )
        return

    if (
        _partial_sem_version(copied_version) <
        _partial_sem_version(original_version)
    ):
        sys.stdout.write(bordered(
            'Configuration needs update (%s < %s). Tip: use "dodo diff ."\n'
            % (
                copied_version,
                original_version,
            )
        ))
        sys.stdout.write('\n')


if Dodo.is_main(__name__, safe=False):
    args = _args()
    if args.check_dodo:
        check_dodo_commands_version()
    if args.check_config:
        check_config_version()
